from dataclasses import dataclass
from typing import Iterable

import typer

from recite.console import ReciteConsole
from recite.step import BumpVersionStep, DynamicVersionDescriptionGitStep, Step


@dataclass(kw_only=True)
class StepRunner:
    beginning_message: str
    console: ReciteConsole
    steps: Iterable[Step]

    def pre_run(self):
        pass  # pragma: no cover

    def post_run(self):
        pass  # pragma: no cover

    def run_steps(self) -> bool:
        self.pre_run()
        self.console.print_message(message=self.beginning_message)
        for number, step in enumerate(self.steps, start=1):
            result = step.run()
            if result.success:
                self.console.print_success(message=step.description, number=number)
            else:
                self.console.print_failure(message=step.description, number=number)
                if result.messages is not None:
                    self.console.print_multiple_messages(
                        messages=result.messages, indent_count=1, color="bad"
                    )
                return False
            if result.messages is not None:
                self.console.print_multiple_messages(
                    messages=result.messages, indent_count=1, color="good"
                )
        self.post_run()
        return True


@dataclass(kw_only=True)
class CheckStepRunner(StepRunner):
    beginning_message: str = (
        ":eyes: Checking everything to make sure you are ready to release :eyes:"
    )

    def post_run(self):
        self.console.print_message(
            message=":nerd_face: Everything looks perfect! :nerd_face:"
        )


@dataclass(kw_only=True)
class PerformReleaseRunner(StepRunner):
    beginning_message: str = ":sparkles: Performing release :sparkles:"
    is_initial: bool = False

    def pre_run(self):
        self.console.print_message("I will perform the following steps:")
        new_version = "0.1.0" if self.is_initial else ""
        for step in self.steps:
            msg = step.description
            if isinstance(step, BumpVersionStep):
                result = step.run(dry_run=True)
                msg = result.messages[0]
                new_version = result.return_value.new_version
            elif isinstance(step, DynamicVersionDescriptionGitStep):
                step.new_version = new_version
                # update msg with updated description
                msg = step.description
            self.console.print_message(message=msg, indent_count=1)
        proceed = typer.confirm("Do you want to proceed?")
        if not proceed:
            raise typer.Abort()

    def post_run(self):
        self.console.print_message(
            message=":rocket: Congrats to your release! :rocket:"
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['recite']

package_data = \
{'': ['*']}

modules = \
['py']
install_requires = \
['gitpython>=3.1.30,<4.0.0',
 'rich>=13.1.0,<14.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer>=0.7.0,<0.8.0']

extras_require = \
{'docs': ['mkdocs>=1.4.2,<2.0.0', 'mkdocs-material>=9.0.9,<10.0.0']}

entry_points = \
{'console_scripts': ['recite = recite.main:app']}

setup_kwargs = {
    'name': 'recite',
    'version': '0.1.0',
    'description': 'Publish your poetry-based projects, without missing important steps',
    'long_description': '<p align="center">\n<img src="https://github.com/dobraczka/recite/raw/main/docs/assets/logo.png" alt="recite logo", width=200/>\n</p>\n<h2 align="center"> recite</h2>\n\n\n<p align="center">\n<a href="https://github.com/dobraczka/recite/actions/workflows/main.yml"><img alt="Actions Status" src="https://github.com/dobraczka/recite/actions/workflows/main.yml/badge.svg?branch=main"></a>\n<a href=\'https://recite.readthedocs.io/en/latest/?badge=latest\'><img src=\'https://readthedocs.org/projects/recite/badge/?version=latest\' alt=\'Documentation Status\' /></a>\n<a href="https://codecov.io/gh/dobraczka/recite"><img src="https://codecov.io/gh/dobraczka/recite/branch/main/graph/badge.svg?token=TCMKS9U0MH"/></a>\n<a href="https://pypi.org/project/recite"/><img alt="Stable python versions" src="https://img.shields.io/pypi/pyversions/recite"></a>\n<a href="https://github.com/psf/black"><img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n</p>\n\n`recite` intends to make releasing [poetry](https://python-poetry.org/)-based libraries easier and avoid missing important steps (e.g. adapting the changelog (because [you should keep one](https://keepachangelog.com/))).\n\n```console\n$ recite patch\n\nrecite > 👀 Checking everything to make sure you are ready to release 👀\nrecite > 1: ✓ Make sure you have a (non-empty) pyproject.toml\nrecite > 2: ✓ Make sure you\'re on main/master branch\nrecite > 3: ✓ Make sure git is clean\nrecite > 4: ✓ Run test-suite\nrecite > 5: ✓ Make sure changelog was updated\nrecite > 🤓 Everything looks perfect! 🤓\nrecite > I will perform the following steps:\nrecite >        * Would bump version from 0.1.0 to 0.1.1\nrecite >        * Commit version bump\nrecite >        * Create git tag 0.1.1\nrecite >        * Push git tag 0.1.1\nrecite >        * Remind you to upload build as github release\nDo you want to proceed? [y/N]: y\nrecite > ✨ Performing release ✨\nrecite > 1: ✓ Bump version\nrecite >        * Bumped version from 0.1.0 to 0.1.1\nrecite > 2: ✓ Commit version bump\nrecite > 3: ✓ Create git tag 0.1.1\nrecite > 4: ✓ Push git tag 0.1.1\nrecite > 5: ✓ Build and publish with poetry\nPlease create a github release now! Did you do it? [y/N]: y\nrecite > 6: ✓ Remind you to upload build as github release\nrecite > 🚀 Congrats to your release! 🚀\n```\n\n# Installation\n\n```console\n$ pip install recite\n```\n\n# Why?\n\nPreviously I used a github action to automatically build and publish a new version of a library if a new tag was pushed. However, sometimes I forgot to something crucial (e.g. to adapt the changelog). In this case I had to rush to stop the github action before it would publish the release to pypi (where it would lie forever unable to be rectified).\nThis ensures all the necessary checks are in place before any tags are created.\n',
    'author': 'Daniel Obraczka',
    'author_email': 'obraczka@informatik.uni-leipzig.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dobraczka/recite',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import os

import typer

from .console import ReciteConsole
from .runner import CheckStepRunner, PerformReleaseRunner
from .step import (
    BumpVersionStep,
    CheckChangelogStep,
    CheckCleanGitStep,
    CheckOnMainStep,
    CheckPyProjectStep,
    CommitVersionBumpStep,
    GithubReleaseReminderStep,
    GitTagStep,
    PoetryPublishStep,
    PushTagStep,
    RunTestsStep,
)

console = ReciteConsole()
app = typer.Typer()


@app.command()
def main(
    release_type: str = typer.Argument(
        ..., help="What type of release is this? For initial release use 'initial'"
    ),
    allow_untracked_files: bool = typer.Option(
        False, help="Allow files not tracked by git"
    ),
    remote: str = typer.Option("origin", help="Where should the tag be pushed?"),
    commit_message: str = typer.Option(
        "Bumped version", help="Commit message for version bump"
    ),
    git_tag_prefix: str = typer.Option("", help="Prefix for git tag"),
):
    project_dir = os.getcwd()
    console = ReciteConsole()
    successful = CheckStepRunner(
        steps=[
            CheckPyProjectStep(),
            CheckOnMainStep(project_dir=project_dir),
            CheckCleanGitStep(
                project_dir=project_dir, allow_untracked_files=allow_untracked_files
            ),
            RunTestsStep(),
            CheckChangelogStep(project_dir=project_dir),
        ],
        console=console,
    ).run_steps()
    if not successful:
        typer.Exit(code=1)
    else:
        is_initial = False
        if release_type == "initial":
            steps = [
                GitTagStep(),
                PushTagStep(remote=remote),
                PoetryPublishStep(),
                GithubReleaseReminderStep(),
            ]
            is_initial = True
        else:
            steps = [
                BumpVersionStep(bump_rule=release_type),
                CommitVersionBumpStep(commit_message=commit_message),
                GitTagStep(),
                PushTagStep(remote=remote),
                PoetryPublishStep(),
                GithubReleaseReminderStep(),
            ]
        PerformReleaseRunner(
            steps=steps,
            console=console,
            is_initial=is_initial,
        ).run_steps()


if __name__ == "__main__":
    app()
